@@ -1,10 +1,10 @@
-#!/usr/bin/python
+#!/usr/bin/env python
+# encoding: utf-8
 
 import sys
 import os.path
-import random
-
-# for example: ./genbench.py /tmp/build 50 100 15 5
+from random import Random
+random = Random(0) # initialise with seed to have reproductible benches
 
 HELP_USAGE = """Usage: generate_libs.py root libs classes internal external.
     root     - Root directory where to create libs.
@@ -13,6 +13,12 @@ HELP_USAGE = """Usage: generate_libs.py root libs classes internal external.
     internal - Number of includes per file referring to that same library
     external - Number of includes per file pointing to other libraries
 
+For example:
+./genbench.py /tmp/build 200 100 15 5
+
+To try the waf part, do:
+waf configure build -p -j5
+
 To test the autotools part, do:
 touch README AUTHORS NEWS ChangeLog &&
 autoreconf --install --symlink --verbose &&
@@ -26,9 +32,9 @@ time make -j4 --silent
 def lib_name(i):
     return "lib_" + str(i)
 
-def CreateHeader(name):
+def createHeader(name):
     filename = name + ".h"
-    handle = file(filename, "w" )
+    handle = open(filename, "w" )
 
     guard = name + '_h_'
     handle.write ('#ifndef ' + guard + '\n');
@@ -43,20 +49,20 @@ def CreateHeader(name):
     handle.write ('#endif\n');
 
 
-def CreateCPP(name, lib_number, classes_per_lib, internal_includes, external_includes):
+def createCPP(name, lib_number, classes_per_lib, internal_includes, external_includes):
     filename = name + ".cpp"
-    handle = file(filename, "w" )
+    handle = open(filename, "w" )
 
     header= name + ".h"
     handle.write ('#include "' + header + '"\n');
 
-    includes = random.sample(xrange(classes_per_lib), internal_includes)
+    includes = random.sample(range(classes_per_lib), internal_includes)
     for i in includes:
         handle.write ('#include "class_' + str(i) + '.h"\n')
 
     if (lib_number > 0):
-        includes = random.sample(xrange(classes_per_lib), external_includes)
-        lib_list = xrange(lib_number)
+        includes = random.sample(range(classes_per_lib), external_includes)
+        lib_list = range(lib_number)
         for i in includes:
             libname = 'lib_' + str(random.choice(lib_list))
             handle.write ('#include <' + libname + '/' + 'class_' + str(i) + '.h>\n')
@@ -66,17 +72,21 @@ def CreateCPP(name, lib_number, classes_per_lib, internal_includes, external_inc
     handle.write (name + '::~' + name  + '() {}\n');
 
 
-def CreateSConscript(lib_number, classes):
-    handle = file("SConscript", "w");
+def createSConscript(lib_number, classes):
+    handle = open("SConscript", "w");
     handle.write("Import('env')\n")
     handle.write('list = Split("""\n');
-    for i in xrange(classes):
+    for i in range(classes):
         handle.write('    class_' + str(i) + '.cpp\n')
     handle.write('    """)\n\n')
     handle.write('env.StaticLibrary("lib_' + str(lib_number) + '", list)\n\n')
 
-def CreateLibMakefile(lib_number, classes):
-    handle = file("Makefile", "w");
+def createLibCMakeLists(lib_number, classes):
+    handle = open("CMakeLists.txt", "w")
+    handle.write("""add_library(lib_%s STATIC %s)\n""" % (str(lib_number), ' '.join(('class_%s' % str(i) for i in range(classes)))))
+
+def createLibMakefile(lib_number, classes):
+    handle = open("Makefile", "w");
     handle.write ("""COMPILER = g++
 INC = -I..
 CCFLAGS = -g -Wall $(INC)
@@ -87,7 +97,7 @@ DEPEND = makedepend
 """)
     handle.write ("lib = lib_" + str(lib_number) + ".a\n")
     handle.write ("src = \\\n")
-    for i in xrange(classes):
+    for i in range(classes):
         handle.write('class_' + str(i) + '.cpp \\\n')
     handle.write ("""
 
@@ -110,17 +120,17 @@ depend:
 
 """)
 
-def CreateLibJamFile(lib_number, classes):
-    handle = file("Jamfile", "w")
+def createLibJamFile(lib_number, classes):
+    handle = open("Jamfile", "w")
     handle.write ("SubDir TOP lib_" + str(lib_number) + " ;\n\n")
     handle.write ("SubDirHdrs $(INCLUDES) ;\n\n")
     handle.write ("Library lib_" + str(lib_number) + " :\n")
-    for i in xrange(classes):
+    for i in range(classes):
         handle.write('    class_' + str(i) + '.cpp\n')
     handle.write ('    ;\n')
 
-def CreateVCProjFile(lib_number, classes):
-    handle = file("lib_" + str(lib_number) + ".vcproj", "w")
+def createVCProjFile(lib_number, classes):
+    handle = open("lib_" + str(lib_number) + ".vcproj", "w")
     handle.write("""<?xml version="1.0" encoding="Windows-1252"?>
 <VisualStudioProject
 	ProjectType="Visual C++"
@@ -163,7 +173,7 @@ def CreateVCProjFile(lib_number, classes):
 	<Files>
 """)
 
-    for i in xrange(classes):
+    for i in range(classes):
         handle.write('  <File RelativePath=".\class_' + str(i) + '.cpp"/>\n')
 
     handle.write("""
@@ -173,37 +183,46 @@ def CreateVCProjFile(lib_number, classes):
 </VisualStudioProject>
 """)
 
-def CreateLibrary(lib_number, classes, internal_includes, external_includes):
+def createLibrary(lib_number, classes, internal_includes, external_includes):
     name = "lib_" + str(lib_number)
-    SetDir(name)
-    for i in xrange(classes):
+    setDir(name)
+    for i in range(classes):
         classname = "class_" + str(i)
-        CreateHeader(classname)
-        CreateCPP(classname, lib_number, classes, internal_includes, external_includes)
-    CreateSConscript(lib_number, classes)
-    CreateLibMakefile(lib_number, classes)
-    #CreateLibJamFile(lib_number, classes)
-    #CreateVCProjFile(lib_number, classes)
-    CreateW(lib_number, classes)
-    CreateAutotools(lib_number, classes)
+        createHeader(classname)
+        createCPP(classname, lib_number, classes, internal_includes, external_includes)
+    createSConscript(lib_number, classes)
+    createLibCMakeLists(lib_number, classes)
+    createLibMakefile(lib_number, classes)
+    createAutotools(lib_number, classes)
 
     os.chdir("..")
 
-def CreateSConstruct(libs):
-    handle = file("SConstruct", "w");
+def createCMakeLists(libs):
+    handle = open("CMakeLists.txt", "w")
+    handle.write("""project('profiling-test')
+cmake_minimum_required(VERSION 2.8)
+
+include_directories(${CMAKE_SOURCE_DIR})
+""")
+
+    for i in range(libs):
+        handle.write("""add_subdirectory(lib_%s)\n""" % str(i))
+
+def createSConstruct(libs):
+    handle = open("SConstruct", "w");
     handle.write("""env = Environment(CPPFLAGS=['-Wall'], CPPDEFINES=['LINUX'], CPPPATH=[Dir('#')])\n""")
     handle.write("""env.Decider('timestamp-newer')\n""")
     handle.write("""env.SetOption('implicit_cache', True)\n""")
     handle.write("""env.SourceCode('.', None)\n""")
 
-    for i in xrange(libs):
+    for i in range(libs):
         handle.write("""env.SConscript("lib_%s/SConscript", exports=['env'])\n""" % str(i))
 
-def CreateFullMakefile(libs):
-    handle = file("Makefile", "w")
+def createFullMakefile(libs):
+    handle = open("Makefile", "w")
 
     handle.write('subdirs = \\\n')
-    for i in xrange(libs):
+    for i in range(libs):
         handle.write('lib_' + str(i) + '\\\n')
     handle.write("""
 
@@ -220,64 +239,57 @@ depend:
 	(cd $$i; $(MAKE) depend); done
 """)
 
-def CreateFullJamfile(libs):
-    handle = file("Jamfile", "w")
+def createFullJamfile(libs):
+    handle = open("Jamfile", "w")
     handle.write ("SubDir TOP ;\n\n")
 
-    for i in xrange(libs):
+    for i in range(libs):
         handle.write('SubInclude TOP ' + lib_name(i) + ' ;\n')
 
-    handle = file("Jamrules", "w")
+    handle = open("Jamrules", "w")
     handle.write('INCLUDES = $(TOP) ;\n')
 
-def CreateW(lib_number, classes):
-    handle = file("wscript", "w");
-    handle.write("def build(bld):\n")
-    handle.write("    obj = bld.new_task_gen('cxx', 'staticlib')\n")
-    handle.write("    obj.includes='..'\n")
-    handle.write("    obj.source='''\n")
+WT = """#! /usr/bin/env python
+# encoding: utf-8
 
-    for i in xrange(classes):
-        handle.write('    class_' + str(i) + '.cpp\n')
+VERSION = '0.0.2'
+APPNAME = 'build_bench'
+top  = '.'
+out  = 'out'
 
-    handle.write("    '''\n")
-    handle.write("    obj.target = 'lib2'\n")
-    handle.write('def set_options(opt): pass\n')
-    handle.write('def configure(conf): pass\n\n')
+def options(opt):
+	opt.load('compiler_cxx')
 
-def CreateWtop(libs):
-    handle = file("wscript", "w")
+def configure(conf):
+	conf.load('compiler_cxx')
 
-    handle.write("VERSION='0.0.1'\n")
-    handle.write("APPNAME='build-bench'\n")
-    handle.write("srcdir = '.'\n")
-    handle.write("blddir = 'build'\n")
+def build(bld):
+	for i in range(%d):
+		filez = ' '.join(['lib_%%d/class_%%d.cpp' %% (i, j) for j in range(%d)])
+		bld.stlib(
+			source = filez,
+			target = 'lib_%%d' %% i,
+			includes = '.', # include the top-level
+		)
+"""
 
-    handle.write("def build(bld):\n")
-    handle.write("    bld.add_subdirs('''\n")
+def createWtop(libs, classes):
+	f = open('wscript', 'w')
+	f.write(WT % (libs, classes))
+	f.close()
 
-    for i in xrange(libs):
-        handle.write("""lib_%s\n""" % str(i))
-
-    handle.write("    ''')\n")
-
-    handle.write("def configure(conf):\n")
-    handle.write("    conf.check_tool('g++')\n")
-    handle.write("def set_options(opt):\n")
-    handle.write("    pass\n\n")
-
-def CreateFullSolution(libs):
-    handle = file("solution.sln", "w")
+def createFullSolution(libs):
+    handle = open("solution.sln", "w")
     handle.write("Microsoft Visual Studio Solution File, Format Version 8.00\n")
 
-    for i in xrange(libs):
+    for i in range(libs):
         project_name = lib_name(i) + '\\' + lib_name(i) + '.vcproj'
         handle.write('Project("{8BC9CEB8-8B4A-11D0-8D11-00A0C91BC942}") = "' + lib_name(i) +
                       '", "' + project_name + '", "{CF495178-8865-4D20-939D-AAA' + str(i) + '}"\n')
         handle.write('EndProject\n')
 
-def CreateAutotoolsTop(libs):
-    handle = file("configure.ac", "w")
+def createAutotoolsTop(libs):
+    handle = open("configure.ac", "w")
     handle.write('''\
 AC_INIT([bench], [1.0.0])
 AC_CONFIG_AUX_DIR([autotools-aux])
@@ -288,30 +300,30 @@ AC_CONFIG_FILES([Makefile])
 AC_OUTPUT
 ''')
 
-    handle = file("Makefile.am", "w")
+    handle = open("Makefile.am", "w")
     handle.write('''\
 AM_CPPFLAGS = -I$(srcdir)
 lib_LTLIBRARIES =
 ''')
-    for i in xrange(libs): handle.write('include lib_%s/Makefile.am\n' % str(i))
+    for i in range(libs): handle.write('include lib_%s/Makefile.am\n' % str(i))
 
-def CreateAutotools(lib_number, classes):
+def createAutotools(lib_number, classes):
 
-    handle = file("Makefile.am", "w")
+    handle = open("Makefile.am", "w")
     handle.write('''\
 lib_LTLIBRARIES += lib%s.la
 lib%s_la_SOURCES =''' % (str(lib_number), str(lib_number)))
-    for i in xrange(classes): handle.write(' lib_%s/class_%s.cpp' % (str(lib_number), str(i)))
+    for i in range(classes): handle.write(' lib_%s/class_%s.cpp' % (str(lib_number), str(i)))
     handle.write('\n')
 
-def SetDir(dir):
+def setDir(dir):
     if (not os.path.exists(dir)):
         os.mkdir(dir)
     os.chdir(dir)
 
 def main(argv):
     if len(argv) != 6:
-        print HELP_USAGE
+        print(HELP_USAGE)
         return
 
     root_dir = argv[1]
@@ -320,16 +332,15 @@ def main(argv):
     internal_includes = int(argv[4])
     external_includes = int(argv[5])
 
-    SetDir(root_dir)
-    for i in xrange(libs):
-        CreateLibrary(i, classes, internal_includes, external_includes)
+    setDir(root_dir)
+    for i in range(libs):
+        createLibrary(i, classes, internal_includes, external_includes)
 
-    CreateSConstruct(libs)
-    CreateFullMakefile(libs)
-    CreateWtop(libs)
-    #CreateFullJamfile(libs)
-    #CreateFullSolution(libs)
-    CreateAutotoolsTop(libs)
+    createSConstruct(libs)
+    createCMakeLists(libs)
+    createFullMakefile(libs)
+    createWtop(libs, classes)
+    createAutotoolsTop(libs)
 
 if __name__ == "__main__":
     main( sys.argv )
