@@ -1,9 +1,9 @@
 #!/usr/bin/env python
 # encoding: utf-8
-# Thomas Nagy, 2005-2010 (ita)
+# Thomas Nagy, 2005-2016 (ita)
 
 """
-Node: filesystem structure, contains lists of nodes
+Node: filesystem structure
 
 #. Each file/folder is represented by exactly one node.
 
@@ -11,13 +11,14 @@ Node: filesystem structure, contains lists of nodes
    Unused class members can increase the `.wafpickle` file size sensibly.
 
 #. Node objects should never be created directly, use
-   the methods :py:func:`Node.make_node` or :py:func:`Node.find_node`
+   the methods :py:func:`Node.make_node` or :py:func:`Node.find_node` for the low-level operations
 
-#. The methods :py:func:`Node.find_resource`, :py:func:`Node.find_dir` :py:func:`Node.find_or_declare` should be
+#. The methods :py:func:`Node.find_resource`, :py:func:`Node.find_dir` :py:func:`Node.find_or_declare` must be
    used when a build context is present
 
-#. Each instance of :py:class:`waflib.Context.Context` has a unique :py:class:`Node` subclass.
-   (:py:class:`waflib.Node.Nod3`, see the :py:class:`waflib.Context.Context` initializer). A reference to the context owning a node is held as self.ctx
+#. Each instance of :py:class:`waflib.Context.Context` has a unique :py:class:`Node` subclass required for serialization.
+   (:py:class:`waflib.Node.Nod3`, see the :py:class:`waflib.Context.Context` initializer). A reference to the context
+   owning a node is held as *self.ctx*
 """
 
 import os, re, sys, shutil
@@ -52,163 +53,234 @@ exclude_regs = '''
 **/{arch}
 **/_darcs
 **/_darcs/**
+**/.intlcache
 **/.DS_Store'''
 """
 Ant patterns for files and folders to exclude while doing the
 recursive traversal in :py:meth:`waflib.Node.Node.ant_glob`
 """
 
-# TODO optimize split_path by performing a replacement when unpacking?
-
-def split_path(path):
-	"""
-	Split a path by os.sep (This is not os.path.split)
-
-	:param path: path to split
-	:type path: string
-	:rtype: list of string
-	:return: the path, split
-	"""
-	return path.split('/')
-
-def split_path_cygwin(path):
-	if path.startswith('//'):
-		ret = path.split('/')[2:]
-		ret[0] = '/' + ret[0]
-		return ret
-	return path.split('/')
-
-re_sp = re.compile('[/\\\\]')
-def split_path_win32(path):
-	if path.startswith('\\\\'):
-		ret = re.split(re_sp, path)[2:]
-		ret[0] = '\\' + ret[0]
-		return ret
-	return re.split(re_sp, path)
-
-if sys.platform == 'cygwin':
-	split_path = split_path_cygwin
-elif Utils.is_win32:
-	split_path = split_path_win32
-
 class Node(object):
 	"""
-	This class is organized in two parts
+	This class is organized in two parts:
 
 	* The basic methods meant for filesystem access (compute paths, create folders, etc)
 	* The methods bound to a :py:class:`waflib.Build.BuildContext` (require ``bld.srcnode`` and ``bld.bldnode``)
 	"""
 
-	__slots__ = ('name', 'sig', 'children', 'parent', 'cache_abspath', 'cache_isdir')
+	dict_class = dict
+	"""
+	Subclasses can provide a dict class to enable case insensitivity for example.
+	"""
+
+	__slots__ = ('name', 'parent', 'children', 'cache_abspath', 'cache_isdir')
 	def __init__(self, name, parent):
+		"""
+		.. note:: Use :py:func:`Node.make_node` or :py:func:`Node.find_node` instead of calling this constructor
+		"""
 		self.name = name
 		self.parent = parent
-
 		if parent:
 			if name in parent.children:
 				raise Errors.WafError('node %s exists in the parent files %r already' % (name, parent))
 			parent.children[name] = self
 
 	def __setstate__(self, data):
-		"Deserializes from data"
+		"Deserializes node information, used for persistence"
 		self.name = data[0]
 		self.parent = data[1]
 		if data[2] is not None:
-			self.children = data[2]
-		if data[3] is not None:
-			self.sig = data[3]
+			# Issue 1480
+			self.children = self.dict_class(data[2])
 
 	def __getstate__(self):
-		"Serialize the node info"
-		return (self.name, self.parent, getattr(self, 'children', None), getattr(self, 'sig', None))
+		"Serializes node information, used for persistence"
+		return (self.name, self.parent, getattr(self, 'children', None))
 
 	def __str__(self):
-		"String representation (name), for debugging purposes"
-		return self.name
+		"""
+		String representation (abspath), for debugging purposes
 
-	def __repr__(self):
-		"String representation (abspath), for debugging purposes"
+		:rtype: string
+		"""
 		return self.abspath()
 
-	def __hash__(self):
-		"Node hash, used for storage in dicts. This hash is not persistent."
-		return id(self)
+	def __repr__(self):
+		"""
+		String representation (abspath), for debugging purposes
 
-	def __eq__(self, node):
-		"Node comparison, based on the IDs"
-		return id(self) == id(node)
+		:rtype: string
+		"""
+		return self.abspath()
 
 	def __copy__(self):
-		"Implemented to prevent nodes from being copied (raises an exception)"
+		"""
+		Provided to prevent nodes from being copied
+
+		:raises: :py:class:`waflib.Errors.WafError`
+		"""
 		raise Errors.WafError('nodes are not supposed to be copied')
 
-	def read(self, flags='r'):
+	def read(self, flags='r', encoding='ISO8859-1'):
 		"""
-		Return the contents of the file represented by this node::
+		Reads and returns the contents of the file represented by this node, see :py:func:`waflib.Utils.readf`::
 
 			def build(bld):
 				bld.path.find_node('wscript').read()
 
-		:type  fname: string
-		:param fname: Path to file
-		:type  m: string
-		:param m: Open mode
-		:rtype: string
+		:param flags: Open mode
+		:type  flags: string
+		:param encoding: encoding value for Python3
+		:type encoding: string
+		:rtype: string or bytes
 		:return: File contents
 		"""
-		return Utils.readf(self.abspath(), flags)
+		return Utils.readf(self.abspath(), flags, encoding)
 
-	def write(self, data, flags='w'):
+	def write(self, data, flags='w', encoding='ISO8859-1'):
 		"""
-		Write some text to the physical file represented by this node::
+		Writes data to the file represented by this node, see :py:func:`waflib.Utils.writef`::
 
 			def build(bld):
 				bld.path.make_node('foo.txt').write('Hello, world!')
 
-		:type  data: string
 		:param data: data to write
-		:type  flags: string
+		:type  data: string
 		:param flags: Write mode
+		:type  flags: string
+		:param encoding: encoding value for Python3
+		:type encoding: string
 		"""
-		f = None
-		try:
-			f = open(self.abspath(), flags)
-			f.write(data)
-		finally:
-			if f:
-				f.close()
+		Utils.writef(self.abspath(), data, flags, encoding)
+
+	def read_json(self, convert=True, encoding='utf-8'):
+		"""
+		Reads and parses the contents of this node as JSON (Python ≥ 2.6)::
+
+			def build(bld):
+				bld.path.find_node('abc.json').read_json()
+
+		Note that this by default automatically decodes unicode strings on Python2, unlike what the Python JSON module does.
+
+		:type  convert: boolean
+		:param convert: Prevents decoding of unicode strings on Python2
+		:type  encoding: string
+		:param encoding: The encoding of the file to read. This default to UTF8 as per the JSON standard
+		:rtype: object
+		:return: Parsed file contents
+		"""
+		import json # Python 2.6 and up
+		object_pairs_hook = None
+		if convert and sys.hexversion < 0x3000000:
+			try:
+				_type = unicode
+			except NameError:
+				_type = str
+
+			def convert(value):
+				if isinstance(value, list):
+					return [convert(element) for element in value]
+				elif isinstance(value, _type):
+					return str(value)
+				else:
+					return value
+
+			def object_pairs(pairs):
+				return dict((str(pair[0]), convert(pair[1])) for pair in pairs)
+
+			object_pairs_hook = object_pairs
+
+		return json.loads(self.read(encoding=encoding), object_pairs_hook=object_pairs_hook)
+
+	def write_json(self, data, pretty=True):
+		"""
+		Writes a python object as JSON to disk (Python ≥ 2.6) as UTF-8 data (JSON standard)::
+
+			def build(bld):
+				bld.path.find_node('xyz.json').write_json(199)
+
+		:type  data: object
+		:param data: The data to write to disk
+		:type  pretty: boolean
+		:param pretty: Determines if the JSON will be nicely space separated
+		"""
+		import json # Python 2.6 and up
+		indent = 2
+		separators = (',', ': ')
+		sort_keys = pretty
+		newline = os.linesep
+		if not pretty:
+			indent = None
+			separators = (',', ':')
+			newline = ''
+		output = json.dumps(data, indent=indent, separators=separators, sort_keys=sort_keys) + newline
+		self.write(output, encoding='utf-8')
+
+	def exists(self):
+		"""
+		Returns whether the Node is present on the filesystem
+
+		:rtype: bool
+		"""
+		return os.path.exists(self.abspath())
+
+	def isdir(self):
+		"""
+		Returns whether the Node represents a folder
+
+		:rtype: bool
+		"""
+		return os.path.isdir(self.abspath())
 
 	def chmod(self, val):
 		"""
-		Change file/dir permissions::
+		Changes the file/dir permissions::
 
 			def build(bld):
 				bld.path.chmod(493) # 0755
 		"""
 		os.chmod(self.abspath(), val)
 
-	def delete(self):
-		"""Delete the file/folder physically (but not the node)"""
+	def delete(self, evict=True):
+		"""
+		Removes the file/folder from the filesystem (equivalent to `rm -rf`), and remove this object from the Node tree.
+		Do not use this object after calling this method.
+		"""
 		try:
-			if getattr(self, 'children', None):
-				shutil.rmtree(self.abspath())
-			else:
-				os.unlink(self.abspath())
-		except:
-			pass
+			try:
+				if os.path.isdir(self.abspath()):
+					shutil.rmtree(self.abspath())
+				else:
+					os.remove(self.abspath())
+			except OSError:
+				if os.path.exists(self.abspath()):
+					raise
+		finally:
+			if evict:
+				self.evict()
 
-		try:
-			delattr(self, 'children')
-		except:
-			pass
+	def evict(self):
+		"""
+		Removes this node from the Node tree
+		"""
+		del self.parent.children[self.name]
 
 	def suffix(self):
-		"""Return the file extension"""
+		"""
+		Returns the file rightmost extension, for example `a.b.c.d → .d`
+
+		:rtype: string
+		"""
 		k = max(0, self.name.rfind('.'))
 		return self.name[k:]
 
 	def height(self):
-		"""Depth in the folder hierarchy from the filesystem root or from all the file drives"""
+		"""
+		Returns the depth in the folder hierarchy from the filesystem root or from all the file drives
+
+		:returns: filesystem depth
+		:rtype: integer
+		"""
 		d = self
 		val = -1
 		while d:
@@ -217,22 +289,28 @@ class Node(object):
 		return val
 
 	def listdir(self):
-		"""List the folder contents"""
+		"""
+		Lists the folder contents
+
+		:returns: list of file/folder names ordered alphabetically
+		:rtype: list of string
+		"""
 		lst = Utils.listdir(self.abspath())
 		lst.sort()
 		return lst
 
 	def mkdir(self):
 		"""
-		Create a folder represented by this node, creating intermediate nodes as needed
-		An exception will be raised only when the folder cannot possibly exist there
+		Creates a folder represented by this node. Intermediate folders are created as needed.
+
+		:raises: :py:class:`waflib.Errors.WafError` when the folder is missing
 		"""
-		if getattr(self, 'cache_isdir', None):
+		if self.isdir():
 			return
 
 		try:
 			self.parent.mkdir()
-		except:
+		except OSError:
 			pass
 
 		if self.name:
@@ -241,26 +319,26 @@ class Node(object):
 			except OSError:
 				pass
 
-			if not os.path.isdir(self.abspath()):
-				raise Errors.WafError('Could not create the directory %s' % self.abspath())
+			if not self.isdir():
+				raise Errors.WafError('Could not create the directory %r' % self)
 
 			try:
 				self.children
-			except:
-				self.children = {}
-
-		self.cache_isdir = True
+			except AttributeError:
+				self.children = self.dict_class()
 
 	def find_node(self, lst):
 		"""
-		Find a node on the file system (files or folders), create intermediate nodes as needed
+		Finds a node on the file system (files or folders), and creates the corresponding Node objects if it exists
 
-		:param lst: path
+		:param lst: relative path
 		:type lst: string or list of string
+		:returns: The corresponding Node object or None if no entry was found on the filesystem
+		:rtype: :py:class:´waflib.Node.Node´
 		"""
 
 		if isinstance(lst, str):
-			lst = [x for x in split_path(lst) if x and x != '.']
+			lst = [x for x in Utils.split_path(lst) if x and x != '.']
 
 		cur = self
 		for x in lst:
@@ -269,46 +347,38 @@ class Node(object):
 				continue
 
 			try:
-				if x in cur.children:
-					cur = cur.children[x]
+				ch = cur.children
+			except AttributeError:
+				cur.children = self.dict_class()
+			else:
+				try:
+					cur = ch[x]
 					continue
-			except:
-				cur.children = {}
+				except KeyError:
+					pass
 
 			# optimistic: create the node first then look if it was correct to do so
 			cur = self.__class__(x, cur)
-			try:
-				os.stat(cur.abspath())
-			except:
-				del cur.parent.children[x]
+			if not cur.exists():
+				cur.evict()
 				return None
 
-		ret = cur
-
-		try:
-			os.stat(ret.abspath())
-		except:
-			del ret.parent.children[ret.name]
+		if not cur.exists():
+			cur.evict()
 			return None
 
-		try:
-			while not getattr(cur.parent, 'cache_isdir', None):
-				cur = cur.parent
-				cur.cache_isdir = True
-		except AttributeError:
-			pass
-
-		return ret
+		return cur
 
 	def make_node(self, lst):
 		"""
-		Find or create a node without looking on the filesystem
+		Returns or creates a Node object corresponding to the input path without considering the filesystem.
 
-		:param lst: path
+		:param lst: relative path
 		:type lst: string or list of string
+		:rtype: :py:class:´waflib.Node.Node´
 		"""
 		if isinstance(lst, str):
-			lst = [x for x in split_path(lst) if x and x != '.']
+			lst = [x for x in Utils.split_path(lst) if x and x != '.']
 
 		cur = self
 		for x in lst:
@@ -316,35 +386,38 @@ class Node(object):
 				cur = cur.parent or cur
 				continue
 
-			if getattr(cur, 'children', {}):
-				if x in cur.children:
-					cur = cur.children[x]
-					continue
+			try:
+				cur = cur.children[x]
+			except AttributeError:
+				cur.children = self.dict_class()
+			except KeyError:
+				pass
 			else:
-				cur.children = {}
+				continue
 			cur = self.__class__(x, cur)
 		return cur
 
-	def search(self, lst):
+	def search_node(self, lst):
 		"""
-		Search for a node without looking on the filesystem
+		Returns a Node previously defined in the data structure. The filesystem is not considered.
 
-		:param lst: path
+		:param lst: relative path
 		:type lst: string or list of string
+		:rtype: :py:class:´waflib.Node.Node´ or None if there is no entry in the Node datastructure
 		"""
 		if isinstance(lst, str):
-			lst = [x for x in split_path(lst) if x and x != '.']
+			lst = [x for x in Utils.split_path(lst) if x and x != '.']
 
 		cur = self
-		try:
-			for x in lst:
-				if x == '..':
-					cur = cur.parent or cur
-				else:
+		for x in lst:
+			if x == '..':
+				cur = cur.parent or cur
+			else:
+				try:
 					cur = cur.children[x]
-			return cur
-		except:
-			pass
+				except (AttributeError, KeyError):
+					return None
+		return cur
 
 	def path_from(self, node):
 		"""
@@ -353,12 +426,13 @@ class Node(object):
 			def build(bld):
 				n1 = bld.path.find_node('foo/bar/xyz.txt')
 				n2 = bld.path.find_node('foo/stuff/')
-				n1.path_from(n2) # './bar/xyz.txt'
+				n1.path_from(n2) # '../bar/xyz.txt'
 
 		:param node: path to use as a reference
 		:type node: :py:class:`waflib.Node.Node`
+		:returns: a relative path or an absolute one if that is better
+		:rtype: string
 		"""
-
 		c1 = self
 		c2 = node
 
@@ -378,49 +452,59 @@ class Node(object):
 			c2 = c2.parent
 			c2h -= 1
 
-		while id(c1) != id(c2):
+		while not c1 is c2:
 			lst.append(c1.name)
 			up += 1
 
 			c1 = c1.parent
 			c2 = c2.parent
 
-		for i in range(up):
-			lst.append('..')
-		lst.reverse()
-		return os.sep.join(lst) or '.'
+		if c1.parent:
+			lst.extend(['..'] * up)
+			lst.reverse()
+			return os.sep.join(lst) or '.'
+		else:
+			return self.abspath()
 
 	def abspath(self):
 		"""
-		Absolute path. A cache is kept in the context as ``cache_node_abspath``
+		Returns the absolute path. A cache is kept in the context as ``cache_node_abspath``
+
+		:rtype: string
 		"""
 		try:
 			return self.cache_abspath
-		except:
+		except AttributeError:
 			pass
 		# think twice before touching this (performance + complexity + correctness)
 
-		if os.sep == '/':
-			if not self.parent:
-				val = os.sep
-			elif not self.parent.name:
-				val = os.sep + self.name
-			else:
-				val = self.parent.abspath() + os.sep + self.name
+		if not self.parent:
+			val = os.sep
+		elif not self.parent.name:
+			val = os.sep + self.name
 		else:
+			val = self.parent.abspath() + os.sep + self.name
+		self.cache_abspath = val
+		return val
+
+	if Utils.is_win32:
+		def abspath(self):
+			try:
+				return self.cache_abspath
+			except AttributeError:
+				pass
 			if not self.parent:
 				val = ''
 			elif not self.parent.name:
 				val = self.name + os.sep
 			else:
 				val = self.parent.abspath().rstrip(os.sep) + os.sep + self.name
-
-		self.cache_abspath = val
-		return val
+			self.cache_abspath = val
+			return val
 
 	def is_child_of(self, node):
 		"""
-		Does this node belong to the subtree node?::
+		Returns whether the object belongs to a subtree of the input node::
 
 			def build(bld):
 				node = bld.path.find_node('wscript')
@@ -428,17 +512,18 @@ class Node(object):
 
 		:param node: path to use as a reference
 		:type node: :py:class:`waflib.Node.Node`
+		:rtype: bool
 		"""
 		p = self
 		diff = self.height() - node.height()
 		while diff > 0:
 			diff -= 1
 			p = p.parent
-		return id(p) == id(node)
+		return p is node
 
 	def ant_iter(self, accept=None, maxdepth=25, pats=[], dir=False, src=True, remove=True):
 		"""
-		Semi-private and recursive method used by ant_glob.
+		Recursive method used by :py:meth:`waflib.Node.ant_glob`.
 
 		:param accept: function used for accepting/rejecting a node, returns the patterns that can be still accepted in recursion
 		:type accept: function
@@ -452,17 +537,20 @@ class Node(object):
 		:type src: bool
 		:param remove: remove files/folders that do not exist (True by default)
 		:type remove: bool
+		:returns: A generator object to iterate from
+		:rtype: iterator
 		"""
 		dircont = self.listdir()
 		dircont.sort()
 
 		try:
 			lst = set(self.children.keys())
+		except AttributeError:
+			self.children = self.dict_class()
+		else:
 			if remove:
 				for x in lst - set(dircont):
-					del self.children[x]
-		except:
-			self.children = {}
+					self.children[x].evict()
 
 		for name in dircont:
 			npats = accept(name, pats)
@@ -471,7 +559,7 @@ class Node(object):
 
 				node = self.make_node([name])
 
-				isdir = os.path.isdir(node.abspath())
+				isdir = node.isdir()
 				if accepted:
 					if isdir:
 						if dir:
@@ -480,7 +568,7 @@ class Node(object):
 						if src:
 							yield node
 
-				if getattr(node, 'cache_isdir', None) or isdir:
+				if isdir:
 					node.cache_isdir = True
 					if maxdepth:
 						for k in node.ant_iter(accept=accept, maxdepth=maxdepth - 1, pats=npats, dir=dir, src=src, remove=remove):
@@ -489,7 +577,7 @@ class Node(object):
 
 	def ant_glob(self, *k, **kw):
 		"""
-		This method is used for finding files across folders. It behaves like ant patterns:
+		Finds files across folders:
 
 		* ``**/*`` find all files recursively
 		* ``**/*.class`` find all files ending by .class
@@ -498,13 +586,14 @@ class Node(object):
 		For example::
 
 			def configure(cfg):
-				cfg.path.ant_glob('**/*.cpp') # find all .cpp files
-				cfg.root.ant_glob('etc/*.txt') # using the filesystem root can be slow
+				cfg.path.ant_glob('**/*.cpp') # finds all .cpp files
+				cfg.root.ant_glob('etc/*.txt') # matching from the filesystem root can be slow
 				cfg.path.ant_glob('*.cpp', excl=['*.c'], src=True, dir=False)
 
 		For more information see http://ant.apache.org/manual/dirtasks.html
 
-		The nodes that correspond to files and folders that do not exist will be removed
+		The nodes that correspond to files and folders that do not exist are garbage-collected.
+		To prevent this behaviour in particular when running over the build directory, pass ``remove=False``
 
 		:param incl: ant patterns or list of patterns to include
 		:type incl: string or list of strings
@@ -518,6 +607,10 @@ class Node(object):
 		:type remove: bool
 		:param maxdepth: maximum depth of recursion
 		:type maxdepth: int
+		:param ignorecase: ignore case while matching (False by default)
+		:type ignorecase: bool
+		:returns: The corresponding Nodes
+		:rtype: list of :py:class:`waflib.Node.Node` instances
 		"""
 
 		src = kw.get('src', True)
@@ -525,6 +618,7 @@ class Node(object):
 
 		excl = kw.get('excl', exclude_regs)
 		incl = k and k[0] or kw.get('incl', '**')
+		reflags = kw.get('ignorecase', 0) and re.I
 
 		def to_pat(s):
 			lst = Utils.to_list(s)
@@ -543,9 +637,9 @@ class Node(object):
 						k = '^%s$' % k
 						try:
 							#print "pattern", k
-							accu.append(re.compile(k))
+							accu.append(re.compile(k, flags=reflags))
 						except Exception as e:
-							raise Errors.WafError("Invalid pattern: %s" % k, e)
+							raise Errors.WafError('Invalid pattern: %s' % k, e)
 				ret.append(accu)
 			return ret
 
@@ -572,40 +666,12 @@ class Node(object):
 				nacc = []
 			return [nacc, nrej]
 
-		ret = [x for x in self.ant_iter(accept=accept, pats=[to_pat(incl), to_pat(excl)], maxdepth=25, dir=dir, src=src, remove=kw.get('remove', True))]
+		ret = [x for x in self.ant_iter(accept=accept, pats=[to_pat(incl), to_pat(excl)], maxdepth=kw.get('maxdepth', 25), dir=dir, src=src, remove=kw.get('remove', True))]
 		if kw.get('flat', False):
 			return ' '.join([x.path_from(self) for x in ret])
 
 		return ret
 
-	def find_nodes(self, find_dirs=True, find_files=True, match_fun=lambda x: True):
-		# FIXME not part of the stable API: find_node vs find_nodes? consistency with argument names on other functions?
-		x = """
-		Recursively finds nodes::
-
-			def configure(cnf):
-				cnf.find_nodes()
-
-		:param find_dirs: whether to return directories
-		:param find_files: whether to return files
-		:param match_fun: matching function, taking a node as parameter
-		:rtype generator
-		:return: a generator that iterates over all the requested files
-		"""
-		files = self.listdir()
-		for f in files:
-			node = self.make_node([f])
-			if os.path.isdir(node.abspath()):
-				if find_dirs and match_fun(node):
-					yield node
-				gen = node.find_nodes(find_dirs, find_files, match_fun)
-				for g in gen:
-					yield g
-			else:
-				if find_files and match_fun(node):
-					yield node
-
-
 	# --------------------------------------------------------------------------------
 	# the following methods require the source/build folders (bld.srcnode/bld.bldnode)
 	# using a subclass is a possibility, but is that really necessary?
@@ -613,52 +679,50 @@ class Node(object):
 
 	def is_src(self):
 		"""
-		True if the node is below the source directory
-		note: !is_src does not imply is_bld()
+		Returns True if the node is below the source directory. Note that ``!is_src() ≠ is_bld()``
 
 		:rtype: bool
 		"""
 		cur = self
-		x = id(self.ctx.srcnode)
-		y = id(self.ctx.bldnode)
+		x = self.ctx.srcnode
+		y = self.ctx.bldnode
 		while cur.parent:
-			if id(cur) == y:
+			if cur is y:
 				return False
-			if id(cur) == x:
+			if cur is x:
 				return True
 			cur = cur.parent
 		return False
 
 	def is_bld(self):
 		"""
-		True if the node is below the build directory
-		note: !is_bld does not imply is_src
+		Returns True if the node is below the build directory. Note that ``!is_bld() ≠ is_src()``
 
 		:rtype: bool
 		"""
 		cur = self
-		y = id(self.ctx.bldnode)
+		y = self.ctx.bldnode
 		while cur.parent:
-			if id(cur) == y:
+			if cur is y:
 				return True
 			cur = cur.parent
 		return False
 
 	def get_src(self):
 		"""
-		Return the equivalent src node (or self if not possible)
+		Returns the corresponding Node object in the source directory (or self if not possible)
 
 		:rtype: :py:class:`waflib.Node.Node`
 		"""
 		cur = self
-		x = id(self.ctx.srcnode)
-		y = id(self.ctx.bldnode)
+		x = self.ctx.srcnode
+		y = self.ctx.bldnode
 		lst = []
 		while cur.parent:
-			if id(cur) == y:
+			if cur is y:
 				lst.reverse()
-				return self.ctx.srcnode.make_node(lst)
-			if id(cur) == x:
+				return x.make_node(lst)
+			if cur is x:
 				return self
 			lst.append(cur.name)
 			cur = cur.parent
@@ -666,18 +730,18 @@ class Node(object):
 
 	def get_bld(self):
 		"""
-		Return the equivalent bld node (or self if not possible)
+		Return the corresponding Node object in the build directory (or self if not possible)
 
 		:rtype: :py:class:`waflib.Node.Node`
 		"""
 		cur = self
-		x = id(self.ctx.srcnode)
-		y = id(self.ctx.bldnode)
+		x = self.ctx.srcnode
+		y = self.ctx.bldnode
 		lst = []
 		while cur.parent:
-			if id(cur) == y:
+			if cur is y:
 				return self
-			if id(cur) == x:
+			if cur is x:
 				lst.reverse()
 				return self.ctx.bldnode.make_node(lst)
 			lst.append(cur.name)
@@ -690,57 +754,49 @@ class Node(object):
 
 	def find_resource(self, lst):
 		"""
-		Try to find a declared build node or a source file
+		Use this method in the build phase to find source files corresponding to the relative path given.
 
-		:param lst: path
+		First it looks up the Node data structure to find any declared Node object in the build directory.
+		If None is found, it then considers the filesystem in the source directory.
+
+		:param lst: relative path
 		:type lst: string or list of string
+		:returns: the corresponding Node object or None
+		:rtype: :py:class:`waflib.Node.Node`
 		"""
 		if isinstance(lst, str):
-			lst = [x for x in split_path(lst) if x and x != '.']
+			lst = [x for x in Utils.split_path(lst) if x and x != '.']
 
-		node = self.get_bld().search(lst)
+		node = self.get_bld().search_node(lst)
 		if not node:
-			self = self.get_src()
-			node = self.find_node(lst)
-		try:
-			pat = node.abspath()
-			if os.path.isdir(pat):
-				return None
-		except:
-			pass
+			node = self.get_src().find_node(lst)
+		if node and node.isdir():
+			return None
 		return node
 
 	def find_or_declare(self, lst):
 		"""
-		if 'self' is in build directory, try to return an existing node
-		if no node is found, go to the source directory
-		try to find an existing node in the source directory
-		if no node is found, create it in the build directory
+		Use this method in the build phase to declare output files.
 
-		:param lst: path
+		If 'self' is in build directory, it first tries to return an existing node object.
+		If no Node is found, it tries to find one in the source directory.
+		If no Node is found, a new Node object is created in the build directory, and the
+		intermediate folders are added.
+
+		:param lst: relative path
 		:type lst: string or list of string
 		"""
 		if isinstance(lst, str):
-			lst = [x for x in split_path(lst) if x and x != '.']
+			lst = [x for x in Utils.split_path(lst) if x and x != '.']
 
-		node = self.get_bld().search(lst)
+		node = self.get_bld().search_node(lst)
 		if node:
 			if not os.path.isfile(node.abspath()):
-				node.sig = None
-				try:
-					node.parent.mkdir()
-				except:
-					pass
+				node.parent.mkdir()
 			return node
 		self = self.get_src()
 		node = self.find_node(lst)
 		if node:
-			if not os.path.isfile(node.abspath()):
-				node.sig = None
-				try:
-					node.parent.mkdir()
-				except:
-					pass
 			return node
 		node = self.get_bld().make_node(lst)
 		node.parent.mkdir()
@@ -748,26 +804,26 @@ class Node(object):
 
 	def find_dir(self, lst):
 		"""
-		Search for a folder in the filesystem
+		Searches for a folder on the filesystem (see :py:meth:`waflib.Node.Node.find_node`)
 
-		:param lst: path
+		:param lst: relative path
 		:type lst: string or list of string
+		:returns: The corresponding Node object or None if there is no such folder
+		:rtype: :py:class:`waflib.Node.Node`
 		"""
 		if isinstance(lst, str):
-			lst = [x for x in split_path(lst) if x and x != '.']
+			lst = [x for x in Utils.split_path(lst) if x and x != '.']
 
 		node = self.find_node(lst)
-		try:
-			if not os.path.isdir(node.abspath()):
-				return None
-		except (OSError, AttributeError):
-			# the node might be None, and raise an AttributeError
+		if node and not node.isdir():
 			return None
 		return node
 
 	# helpers for building things
 	def change_ext(self, ext, ext_in=None):
 		"""
+		Declares a build node with a distinct extension; this is uses :py:meth:`waflib.Node.Node.find_or_declare`
+
 		:return: A build node of the same path, but with a different extension
 		:rtype: :py:class:`waflib.Node.Node`
 		"""
@@ -783,60 +839,95 @@ class Node(object):
 
 		return self.parent.find_or_declare([name])
 
-	def nice_path(self, env=None):
-		"""
-		Return the path seen from the launch directory. It is often used for printing nodes in the console to open
-		files easily.
-
-		:param env: unused, left for compatibility with waf 1.5
-		"""
-		return self.path_from(self.ctx.launch_node())
-
 	def bldpath(self):
-		"Path seen from the build directory default/src/foo.cpp"
+		"""
+		Returns the relative path seen from the build directory ``src/foo.cpp``
+
+		:rtype: string
+		"""
 		return self.path_from(self.ctx.bldnode)
 
 	def srcpath(self):
-		"Path seen from the source directory ../src/foo.cpp"
+		"""
+		Returns the relative path seen from the source directory ``../src/foo.cpp``
+
+		:rtype: string
+		"""
 		return self.path_from(self.ctx.srcnode)
 
 	def relpath(self):
-		"If a file in the build directory, bldpath, else srcpath"
+		"""
+		If a file in the build directory, returns :py:meth:`waflib.Node.Node.bldpath`,
+		else returns :py:meth:`waflib.Node.Node.srcpath`
+
+		:rtype: string
+		"""
 		cur = self
-		x = id(self.ctx.bldnode)
+		x = self.ctx.bldnode
 		while cur.parent:
-			if id(cur) == x:
+			if cur is x:
 				return self.bldpath()
 			cur = cur.parent
 		return self.srcpath()
 
 	def bld_dir(self):
-		"Build path without the file name"
+		"""
+		Equivalent to self.parent.bldpath()
+
+		:rtype: string
+		"""
 		return self.parent.bldpath()
 
-	def bld_base(self):
-		"Build path without the extension: src/dir/foo(.cpp)"
-		s = os.path.splitext(self.name)[0]
-		return self.bld_dir() + os.sep + s
+	def h_file(self):
+		"""
+		See :py:func:`waflib.Utils.h_file`
+
+		:return: a hash representing the file contents
+		:rtype: string or bytes
+		"""
+		return Utils.h_file(self.abspath())
 
 	def get_bld_sig(self):
 		"""
-		Node signature, assuming the file is in the build directory
-		"""
-		try:
-			ret = self.ctx.hash_cache[id(self)]
-		except KeyError:
-			pass
-		except AttributeError:
-			self.ctx.hash_cache = {}
-		else:
-			return ret
+		Returns a signature (see :py:meth:`waflib.Node.Node.h_file`) for the purpose
+		of build dependency calculation. This method uses a per-context cache.
 
-		if not self.is_bld() or self.ctx.bldnode is self.ctx.srcnode:
-			self.sig = Utils.h_file(self.abspath())
-		self.ctx.hash_cache[id(self)] = ret = self.sig
+		:return: a hash representing the object contents
+		:rtype: string or bytes
+		"""
+		# previous behaviour can be set by returning self.ctx.node_sigs[self] when a build node
+		try:
+			cache = self.ctx.cache_sig
+		except AttributeError:
+			cache = self.ctx.cache_sig = {}
+		try:
+			ret = cache[self]
+		except KeyError:
+			p = self.abspath()
+			try:
+				ret = cache[self] = self.h_file()
+			except EnvironmentError:
+				if self.isdir():
+					# allow folders as build nodes, do not use the creation time
+					st = os.stat(p)
+					ret = cache[self] = Utils.h_list([p, st.st_ino, st.st_mode])
+					return ret
+				raise
 		return ret
 
+	# --------------------------------------------
+	# TODO waf 2.0, remove the sig and cache_sig attributes
+	def get_sig(self):
+		return self.h_file()
+	def set_sig(self, val):
+		# clear the cache, so that past implementation should still work
+		try:
+			del self.get_bld_sig.__cache__[(self,)]
+		except (AttributeError, KeyError):
+			pass
+	sig = property(get_sig, set_sig)
+	cache_sig = property(get_sig, set_sig)
+
 pickle_lock = Utils.threading.Lock()
 """Lock mandatory for thread-safe node serialization"""
 
