@@ -1,6 +1,6 @@
 #!/usr/bin/env python
 # encoding: utf-8
-# Thomas Nagy, 2006-2010 (ita)
+# Thomas Nagy, 2006-2016 (ita)
 
 """
 Java support
@@ -26,15 +26,14 @@ You would have to run::
 [1] http://www.jython.org/
 """
 
-import os, tempfile, shutil
-from waflib import Task, Utils, Errors, Node, Logs
+import os, shutil
+from waflib import Task, Utils, Errors, Node
 from waflib.Configure import conf
 from waflib.TaskGen import feature, before_method, after_method
 
 from waflib.Tools import ccroot
 ccroot.USELIB_VARS['javac'] = set(['CLASSPATH', 'JAVACFLAGS'])
 
-
 SOURCE_RE = '**/*.java'
 JAR_RE = '**/*'
 
@@ -77,7 +76,7 @@ def apply_java(self):
 		outdir = self.path.get_bld()
 	outdir.mkdir()
 	self.outdir = outdir
-	self.env['OUTDIR'] = outdir.abspath()
+	self.env.OUTDIR = outdir.abspath()
 
 	self.javac_task = tsk = self.create_task('javac')
 	tmp = []
@@ -96,7 +95,7 @@ def apply_java(self):
 	tsk.srcdir = tmp
 
 	if getattr(self, 'compat', None):
-		tsk.env.append_value('JAVACFLAGS', ['-source', self.compat])
+		tsk.env.append_value('JAVACFLAGS', ['-source', str(self.compat)])
 
 	if hasattr(self, 'sourcepath'):
 		fold = [isinstance(x, Node.Node) and x or self.path.find_dir(x) for x in self.to_list(self.sourcepath)]
@@ -111,7 +110,7 @@ def apply_java(self):
 @after_method('apply_java')
 def use_javac_files(self):
 	"""
-	Process the *use* attribute referring to other java compilations
+	Processes the *use* attribute referring to other java compilations
 	"""
 	lst = []
 	self.uselib = self.to_list(getattr(self, 'uselib', []))
@@ -120,21 +119,23 @@ def use_javac_files(self):
 	for x in names:
 		try:
 			y = get(x)
-		except Exception:
+		except Errors.WafError:
 			self.uselib.append(x)
 		else:
 			y.post()
-			lst.append(y.jar_task.outputs[0].abspath())
-			self.javac_task.set_run_after(y.jar_task)
-
-	if lst:
-		self.env.append_value('CLASSPATH', lst)
+			if hasattr(y, 'jar_task'):
+				lst.append(y.jar_task.outputs[0].abspath())
+				self.javac_task.set_run_after(y.jar_task)
+			else:
+				for tsk in y.tasks:
+					self.javac_task.set_run_after(tsk)
+	self.env.append_value('CLASSPATH', lst)
 
 @feature('javac')
 @after_method('apply_java', 'propagate_uselib_vars', 'use_javac_files')
 def set_classpath(self):
 	"""
-	Set the CLASSPATH value on the *javac* task previously created.
+	Sets the CLASSPATH value on the *javac* task previously created.
 	"""
 	self.env.append_value('CLASSPATH', getattr(self, 'classpath', []))
 	for x in self.tasks:
@@ -145,7 +146,7 @@ def set_classpath(self):
 @before_method('process_source')
 def jar_files(self):
 	"""
-	Create a jar task. There can be only one jar task by task generator.
+	Creates a jar task (one maximum per task generator)
 	"""
 	destfile = getattr(self, 'destfile', 'test.jar')
 	jaropts = getattr(self, 'jaropts', [])
@@ -163,7 +164,10 @@ def jar_files(self):
 	self.jar_task = tsk = self.create_task('jar_create')
 	if manifest:
 		jarcreate = getattr(self, 'jarcreate', 'cfm')
-		node = self.path.find_node(manifest)
+		if not isinstance(manifest,Node.Node):
+			node = self.path.find_or_declare(manifest)
+		else:
+			node = manifest
 		tsk.dep_nodes.append(node)
 		jaropts.insert(0, node.abspath())
 	else:
@@ -179,8 +183,8 @@ def jar_files(self):
 	jaropts.append(basedir.bldpath())
 	jaropts.append('.')
 
-	tsk.env['JAROPTS'] = jaropts
-	tsk.env['JARCREATE'] = jarcreate
+	tsk.env.JAROPTS = jaropts
+	tsk.env.JARCREATE = jarcreate
 
 	if getattr(self, 'javac_task', None):
 		tsk.set_run_after(self.javac_task)
@@ -189,7 +193,7 @@ def jar_files(self):
 @after_method('jar_files')
 def use_jar_files(self):
 	"""
-	Process the *use* attribute to set the build order on the
+	Processes the *use* attribute to set the build order on the
 	tasks created by another task generator.
 	"""
 	self.uselib = self.to_list(getattr(self, 'uselib', []))
@@ -198,15 +202,30 @@ def use_jar_files(self):
 	for x in names:
 		try:
 			y = get(x)
-		except Exception:
+		except Errors.WafError:
 			self.uselib.append(x)
 		else:
 			y.post()
 			self.jar_task.run_after.update(y.tasks)
 
-class jar_create(Task.Task):
+class JTask(Task.Task):
 	"""
-	Create a jar file
+	Base class for java and jar tasks; provides functionality to run long commands
+	"""
+	def split_argfile(self, cmd):
+		inline = [cmd[0]]
+		infile = []
+		for x in cmd[1:]:
+			# jar and javac do not want -J flags in @file
+			if x.startswith('-J'):
+				inline.append(x)
+			else:
+				infile.append(self.quote_flag(x))
+		return (inline, infile)
+
+class jar_create(JTask):
+	"""
+	Creates a jar file
 	"""
 	color   = 'GREEN'
 	run_str = '${JAR} ${JARCREATE} ${TGT} ${JAROPTS}'
@@ -227,20 +246,26 @@ class jar_create(Task.Task):
 				raise Errors.WafError('Could not find the basedir %r for %r' % (self.basedir, self))
 		return super(jar_create, self).runnable_status()
 
-class javac(Task.Task):
+class javac(JTask):
 	"""
-	Compile java files
+	Compiles java files
 	"""
 	color   = 'BLUE'
-
+	run_str = '${JAVAC} -classpath ${CLASSPATH} -d ${OUTDIR} ${JAVACFLAGS} ${SRC}'
 	vars = ['CLASSPATH', 'JAVACFLAGS', 'JAVAC', 'OUTDIR']
 	"""
 	The javac task will be executed again if the variables CLASSPATH, JAVACFLAGS, JAVAC or OUTDIR change.
 	"""
+	def uid(self):
+		"""Identify java tasks by input&output folder"""
+		lst = [self.__class__.__name__, self.generator.outdir.abspath()]
+		for x in self.srcdir:
+			lst.append(x.abspath())
+		return Utils.h_list(lst)
 
 	def runnable_status(self):
 		"""
-		Wait for dependent tasks to be complete, then read the file system to find the input nodes.
+		Waits for dependent tasks to be complete, then read the file system to find the input nodes.
 		"""
 		for t in self.run_after:
 			if not t.hasrun:
@@ -253,55 +278,12 @@ class javac(Task.Task):
 				self.inputs.extend(x.ant_glob(SOURCE_RE, remove=False))
 		return super(javac, self).runnable_status()
 
-	def run(self):
-		"""
-		Execute the javac compiler
-		"""
-		env = self.env
-		gen = self.generator
-		bld = gen.bld
-		wd = bld.bldnode.abspath()
-		def to_list(xx):
-			if isinstance(xx, str): return [xx]
-			return xx
-		cmd = []
-		cmd.extend(to_list(env['JAVAC']))
-		cmd.extend(['-classpath'])
-		cmd.extend(to_list(env['CLASSPATH']))
-		cmd.extend(['-d'])
-		cmd.extend(to_list(env['OUTDIR']))
-		cmd.extend(to_list(env['JAVACFLAGS']))
-
-		files = [a.path_from(bld.bldnode) for a in self.inputs]
-
-		# workaround for command line length limit:
-		# http://support.microsoft.com/kb/830473
-		tmp = None
-		try:
-			if len(str(files)) + len(str(cmd)) > 8192:
-				(fd, tmp) = tempfile.mkstemp(dir=bld.bldnode.abspath())
-				try:
-					os.write(fd, '\n'.join(files).encode())
-				finally:
-					if tmp:
-						os.close(fd)
-				if Logs.verbose:
-					Logs.debug('runner: %r' % (cmd + files))
-				cmd.append('@' + tmp)
-			else:
-				cmd += files
-
-			ret = self.exec_command(cmd, cwd=wd, env=env.env or None)
-		finally:
-			if tmp:
-				os.remove(tmp)
-		return ret
-
 	def post_run(self):
 		"""
+		List class files created
 		"""
-		for n in self.generator.outdir.ant_glob('**/*.class'):
-			n.sig = Utils.h_file(n.abspath()) # careful with this
+		for node in self.generator.outdir.ant_glob('**/*.class'):
+			self.generator.bld.node_sigs[node] = self.uid()
 		self.generator.bld.task_sigs[self.uid()] = self.cache_sig
 
 @feature('javadoc')
@@ -317,6 +299,9 @@ def create_javadoc(self):
 		self.javadoc_output = self.bld.path.find_or_declare(self.javadoc_output)
 
 class javadoc(Task.Task):
+	"""
+	Builds java documentation
+	"""
 	color = 'BLUE'
 
 	def __str__(self):
@@ -325,7 +310,7 @@ class javadoc(Task.Task):
 	def run(self):
 		env = self.env
 		bld = self.generator.bld
-		wd = bld.bldnode.abspath()
+		wd = bld.bldnode
 
 		#add src node + bld node (for generated java code)
 		srcpath = self.generator.path.abspath() + os.sep + self.generator.srcdir
@@ -338,7 +323,7 @@ class javadoc(Task.Task):
 		classpath = "".join(classpath)
 
 		self.last_cmd = lst = []
-		lst.extend(Utils.to_list(env['JAVADOC']))
+		lst.extend(Utils.to_list(env.JAVADOC))
 		lst.extend(['-d', self.generator.javadoc_output.abspath()])
 		lst.extend(['-sourcepath', srcpath])
 		lst.extend(['-classpath', classpath])
@@ -350,13 +335,13 @@ class javadoc(Task.Task):
 
 	def post_run(self):
 		nodes = self.generator.javadoc_output.ant_glob('**')
-		for x in nodes:
-			x.sig = Utils.h_file(x.abspath())
+		for node in nodes:
+			self.generator.bld.node_sigs[node] = self.uid()
 		self.generator.bld.task_sigs[self.uid()] = self.cache_sig
 
 def configure(self):
 	"""
-	Detect the javac, java and jar programs
+	Detects the javac, java and jar programs
 	"""
 	# If JAVA_PATH is set, we prepend it to the path list
 	java_path = self.environ['PATH'].split(os.pathsep)
@@ -364,36 +349,37 @@ def configure(self):
 
 	if 'JAVA_HOME' in self.environ:
 		java_path = [os.path.join(self.environ['JAVA_HOME'], 'bin')] + java_path
-		self.env['JAVA_HOME'] = [self.environ['JAVA_HOME']]
+		self.env.JAVA_HOME = [self.environ['JAVA_HOME']]
 
 	for x in 'javac java jar javadoc'.split():
 		self.find_program(x, var=x.upper(), path_list=java_path)
 
 	if 'CLASSPATH' in self.environ:
-		v['CLASSPATH'] = self.environ['CLASSPATH']
+		v.CLASSPATH = self.environ['CLASSPATH']
 
-	if not v['JAR']: self.fatal('jar is required for making java packages')
-	if not v['JAVAC']: self.fatal('javac is required for compiling java classes')
+	if not v.JAR:
+		self.fatal('jar is required for making java packages')
+	if not v.JAVAC:
+		self.fatal('javac is required for compiling java classes')
 
-	v['JARCREATE'] = 'cf' # can use cvf
-	v['JAVACFLAGS'] = []
+	v.JARCREATE = 'cf' # can use cvf
+	v.JAVACFLAGS = []
 
 @conf
 def check_java_class(self, classname, with_classpath=None):
 	"""
-	Check if the specified java class exists
+	Checks if the specified java class exists
 
 	:param classname: class to check, like java.util.HashMap
 	:type classname: string
 	:param with_classpath: additional classpath to give
 	:type with_classpath: string
 	"""
-
 	javatestdir = '.waf-javatest'
 
 	classpath = javatestdir
-	if self.env['CLASSPATH']:
-		classpath += os.pathsep + self.env['CLASSPATH']
+	if self.env.CLASSPATH:
+		classpath += os.pathsep + self.env.CLASSPATH
 	if isinstance(with_classpath, str):
 		classpath += os.pathsep + with_classpath
 
@@ -403,10 +389,10 @@ def check_java_class(self, classname, with_classpath=None):
 	Utils.writef(os.path.join(javatestdir, 'Test.java'), class_check_source)
 
 	# Compile the source
-	self.exec_command(self.env['JAVAC'] + [os.path.join(javatestdir, 'Test.java')], shell=False)
+	self.exec_command(self.env.JAVAC + [os.path.join(javatestdir, 'Test.java')], shell=False)
 
 	# Try to run the app
-	cmd = self.env['JAVA'] + ['-cp', classpath, 'Test', classname]
+	cmd = self.env.JAVA + ['-cp', classpath, 'Test', classname]
 	self.to_log("%s\n" % str(cmd))
 	found = self.exec_command(cmd, shell=False)
 
@@ -419,7 +405,7 @@ def check_java_class(self, classname, with_classpath=None):
 @conf
 def check_jni_headers(conf):
 	"""
-	Check for jni headers and libraries. On success the conf.env variables xxx_JAVA are added for use in C/C++ targets::
+	Checks for jni headers and libraries. On success the conf.env variables xxx_JAVA are added for use in C/C++ targets::
 
 		def options(opt):
 			opt.load('compiler_c')
@@ -431,7 +417,6 @@ def check_jni_headers(conf):
 		def build(bld):
 			bld.shlib(source='a.c', target='app', use='JAVA')
 	"""
-
 	if not conf.env.CC_NAME and not conf.env.CXX_NAME:
 		conf.fatal('load a compiler first (gcc, g++, ..)')
 
@@ -439,7 +424,7 @@ def check_jni_headers(conf):
 		conf.fatal('set JAVA_HOME in the system environment')
 
 	# jni requires the jvm
-	javaHome = conf.env['JAVA_HOME'][0]
+	javaHome = conf.env.JAVA_HOME[0]
 
 	dir = conf.root.find_dir(conf.env.JAVA_HOME[0] + '/include')
 	if dir is None:
@@ -460,6 +445,8 @@ def check_jni_headers(conf):
 	if f:
 		libDirs = [[x, y.parent.abspath()] for x in libDirs for y in f]
 
+	if conf.env.DEST_OS == 'freebsd':
+		conf.env.append_unique('LINKFLAGS_JAVA', '-pthread')
 	for d in libDirs:
 		try:
 			conf.check(header_name='jni.h', define_name='HAVE_JNI_H', lib='jvm',
@@ -471,4 +458,3 @@ def check_jni_headers(conf):
 	else:
 		conf.fatal('could not find lib jvm in %r (see config.log)' % libDirs)
 
-
